@@ -1,5 +1,5 @@
 ## -*- truncate-lines: t; -*-
-## Copyright (C) 2008-18  Enrico Schumann
+## Copyright (C) 2008-19  Enrico Schumann
 
 btest  <- function(prices,
                    signal,
@@ -164,9 +164,12 @@ btest  <- function(prices,
     if (!missing(timestamp) &&
         (inherits(timestamp, "Date") || inherits(timestamp, "POSIXct")) &&
         inherits(b, class(timestamp))) {
-        b <- matchOrNext(b, timestamp)
+        b <- if (b < min(timestamp))
+                 0
+             else
+                 .match_or_previous(b, timestamp)
     }
-            
+
     if ("tradeOnOpen" %in% names(list(...)))
         warning("Did you mean 'trade.at.open'? See ChangeLog 2017-11-14.")
 
@@ -207,7 +210,7 @@ btest  <- function(prices,
 
     db.tc_fun <- if (is.function(tc) && isdebugged(tc))
                        TRUE else FALSE
-    
+
     if (is.null(do.signal) || identical(do.signal, TRUE)) {
         do.signal <- function(...)
             TRUE
